@@ -1,8 +1,8 @@
-/*  $Id: BoolExprParser.cpp,v 1.16 2013/03/03 06:48:45 sarrazip Exp $
+/*  $Id: BoolExprParser.cpp,v 1.18 2016/05/04 04:05:20 sarrazip Exp $
     BoolExprParser.cpp - Boolean expression parser and syntax tree builder
 
     boolstuff - Disjunctive Normal Form boolean expression library
-    Copyright (C) 2002-2008 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2002-2016 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -31,6 +31,24 @@ using namespace std;
 using namespace boolstuff;
 
 
+// Home-made replacement for auto_ptr<>, which is deprecated
+// under gcc 6.1.0.
+//
+template <typename T>
+class AutoPtr
+{
+public:
+    AutoPtr(T *_ptr) : ptr(_ptr) {}
+    ~AutoPtr() { delete ptr; }
+    T *release() { T *ret = ptr; ptr = NULL; return ret; }
+private:
+    AutoPtr(const AutoPtr &);
+    AutoPtr &operator = (const AutoPtr &);
+private:
+    T *ptr;
+};
+
+
 BoolExprParser::BoolExprParser()
   : curInput(),
     curIndex(0)
@@ -49,7 +67,7 @@ BoolExprParser::parse(const std::string &expr) throw(Error)
     curInput = expr;
     curIndex = 0;
 
-    auto_ptr< BoolExpr<string> > root(parseExpr());
+    AutoPtr< BoolExpr<string> > root(parseExpr());
     if (!atEnd())
         throw Error(curIndex, Error::GARBAGE_AT_END);
     return root.release();
@@ -59,7 +77,7 @@ BoolExprParser::parse(const std::string &expr) throw(Error)
 BoolExpr<string> *
 BoolExprParser::parseExpr() throw(Error)
 {
-    auto_ptr< BoolExpr<string> > left(parseTerm());
+    AutoPtr< BoolExpr<string> > left(parseTerm());
 
     if (!tokenSeen("|"))
         return left.release();
@@ -73,7 +91,7 @@ BoolExprParser::parseExpr() throw(Error)
 BoolExpr<string> *
 BoolExprParser::parseTerm() throw(Error)
 {
-    auto_ptr< BoolExpr<string> > left(parseFactor());
+    AutoPtr< BoolExpr<string> > left(parseFactor());
 
     if (!tokenSeen("&"))
         return left.release();
@@ -110,7 +128,7 @@ BoolExprParser::parseAtom() throw(Error)
     if (tokenSeen("("))
     {
         skipToken("(");
-        auto_ptr< BoolExpr<string> > expr(parseExpr());  // may throw
+        AutoPtr< BoolExpr<string> > expr(parseExpr());  // may throw
 
         if (!tokenSeen(")"))
             throw Error(startIndex, Error::RUNAWAY_PARENTHESIS);
@@ -134,7 +152,7 @@ BoolExprParser::parseIdentifier() throw(Error)
     while (curIndex < inputLen && isIdentifierChar(curInput[curIndex]))
         curIndex++;
     if (curIndex == startIndex)
-	throw Error(startIndex, Error::IDENTIFIER_EXPECTED);
+        throw Error(startIndex, Error::IDENTIFIER_EXPECTED);
     string s(curInput, startIndex, curIndex - startIndex);
     return new BoolExpr<string>(s);
 }
