@@ -1,4 +1,4 @@
-/*  $Id: checkc.c,v 1.18 2013/12/08 15:16:12 sarrazip Exp $
+/*  $Id: checkc.c,v 1.20 2015/01/21 01:47:41 sarrazip Exp $
 
     boolstuff - Disjunctive Normal Form boolean expression library
     Copyright (C) 2002-2005 Pierre Sarrazin <http://sarrazip.com/>
@@ -38,13 +38,50 @@
 #define __attribute__(x)
 #endif
 
+
+static void foo(char *str){
+        printf("Converting %s\n", str);
+        size_t errIndex = (size_t) -1;
+        enum bool_error_t errCode = BOOLSTUFF_ERR_GARBAGE_AT_END;
+        printf("Ready to parse\n");
+        boolexpr_t expr = boolstuff_parse(str, &errIndex, &errCode);
+        printf("Ready to get DNF: %d\n", errCode);
+        assert(expr);
+        assert(errCode == BOOLSTUFF_OK);
+        boolexpr_t dnf = boolstuff_get_disjunctive_normal_form(expr);  // modifies tree at 'expr'
+        assert(dnf);
+        expr = NULL;  // tree at 'expr' typically modified; 'expr' not usable anymore
+        printf("Ready to get term roots\n");
+        size_t numRoots = (size_t) -1;
+        boolexpr_t *roots = boolstuff_get_dnf_term_roots(dnf, &numRoots);
+        printf("Found %zu roots\n", numRoots);
+        size_t i;
+        for(i=0; i < numRoots; i++)
+        {
+                char *x = boolstuff_print_tree_to_string(roots[i]);
+                printf("Root[%zu]: %s\n", i, x);
+                boolstuff_free_string(x);
+        }
+        boolstuff_destroy_tree(dnf);
+}
+
+
+int test_foo(){
+        foo("A&B&(C|D)");
+        foo("(A|B)&(A&D)");
+        foo("(A&B)&(C&A)");
+        foo("(A|(B&C))&D");
+        return 0;
+}
+
+
 int
 main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
 {
     boolexpr_t upper_and_node, lower_and_node, or_node, not_node,
-    		a_node, b_node, c_node, d_node,
-		dnf_root, dnf_clone,
-		parsed_tree;
+                a_node, b_node, c_node, d_node,
+                dnf_root, dnf_clone,
+                parsed_tree;
     int in_dnf;
     char *str;
     size_t error_index = 0;
@@ -55,10 +92,10 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
     printf("Checking the C API\n\n");
 
     /*
-	Create a tree for the expression a & !(b | c & d).
-	Some superfluous steps are inserted for testing purposes.
+        Create a tree for the expression a & !(b | c & d).
+        Some superfluous steps are inserted for testing purposes.
 
-	The tree structure is supposed to be this:
+        The tree structure is supposed to be this:
 
               AND
              /   \
@@ -80,10 +117,10 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
     d_node = boolstuff_create_value_node("d");
 
     lower_and_node = boolstuff_create_operator_node(
-					BOOLSTUFF_AND, c_node, d_node);
+                                        BOOLSTUFF_AND, c_node, d_node);
 
     or_node = boolstuff_create_operator_node(
-					BOOLSTUFF_OR, b_node, lower_and_node);
+                                        BOOLSTUFF_OR, b_node, lower_and_node);
     assert(boolstuff_get_node_type(or_node) == BOOLSTUFF_OR);
     assert(boolstuff_get_left_subtree(or_node) == b_node);
     assert(boolstuff_get_right_subtree(or_node) == lower_and_node);
@@ -109,7 +146,7 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
     not_node = boolstuff_create_operator_node(BOOLSTUFF_NOT, NULL, or_node);
 
     upper_and_node = boolstuff_create_operator_node(
-					BOOLSTUFF_AND, a_node, not_node);
+                                        BOOLSTUFF_AND, a_node, not_node);
     assert(boolstuff_get_node_type(upper_and_node) == BOOLSTUFF_AND);
 
     printf("Original expression: ");
@@ -117,7 +154,7 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
     printf("\n");
 
     /*
-	Test the structure of the tree, which is expected to be this:
+        Test the structure of the tree, which is expected to be this:
     */
     assert(LEFT(upper_and_node) != NULL);
     assert(LEFT(LEFT(upper_and_node)) == NULL);
@@ -131,7 +168,7 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
     assert(RIGHT(RIGHT(RIGHT(RIGHT(upper_and_node)))) != NULL);
 
     /*
-	Test the types of the nodes:
+        Test the types of the nodes:
     */
     assert(TYPE(upper_and_node) == BOOLSTUFF_AND);
     assert(TYPE(LEFT(upper_and_node)) == BOOLSTUFF_VALUE);
@@ -154,7 +191,7 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
 
     dnf_root = boolstuff_get_disjunctive_normal_form(upper_and_node);
     assert(dnf_root != NULL);
-		/* result cannot be null is this particular case */
+                /* result cannot be null is this particular case */
 
     str = boolstuff_print_tree_to_string(dnf_root);
     printf("DNF: %s\n", str);
@@ -168,51 +205,51 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
 
 
     /*
-	Test DNF term roots:
+        Test DNF term roots:
     */
     {
-	boolexpr_t *array;
-	size_t array_size, i;
+        boolexpr_t *array;
+        size_t array_size, i;
 
-	array = boolstuff_get_dnf_term_roots(dnf_root, &array_size);
-	assert(array != NULL);
-	assert(array_size > 0);
-	assert(array[array_size] == NULL);
+        array = boolstuff_get_dnf_term_roots(dnf_root, &array_size);
+        assert(array != NULL);
+        assert(array_size > 0);
+        assert(array[array_size] == NULL);
 
-	printf("DNF term roots: %u terms:\n{\n", (unsigned) array_size);
-	for (i = 0; i < array_size; i++)
-	{
-	    char *s;
-	    char **posArray, **negArray;
-	    size_t j;
+        printf("DNF term roots: %u terms:\n{\n", (unsigned) array_size);
+        for (i = 0; i < array_size; i++)
+        {
+            char *s;
+            char **posArray, **negArray;
+            size_t j;
 
-	    assert(array[i] != NULL);
-	    s = boolstuff_print_tree_to_string(array[i]);
-	    printf("Term at index %u: %s\n", (unsigned) i, s);
-	    boolstuff_free_string(s);
+            assert(array[i] != NULL);
+            s = boolstuff_print_tree_to_string(array[i]);
+            printf("Term at index %u: %s\n", (unsigned) i, s);
+            boolstuff_free_string(s);
 
-	    boolstuff_get_tree_variables(array[i], &posArray, &negArray);
+            boolstuff_get_tree_variables(array[i], &posArray, &negArray);
 
-	    printf("\t""Positives: ");
-	    for (j = 0; posArray[j] != NULL; j++)
-		printf("'%s' ", posArray[j]);
-	    printf("\n");
-	    printf("\t""Negatives: ");
-	    for (j = 0; negArray[j] != NULL; j++)
-		printf("'%s' ", negArray[j]);
-	    printf("\n");
-	    
-	    boolstuff_free_variables_sets(posArray, negArray);
-	}
-	assert(array[array_size] == NULL);
-	printf("}\n");
+            printf("\t""Positives: ");
+            for (j = 0; posArray[j] != NULL; j++)
+                printf("'%s' ", posArray[j]);
+            printf("\n");
+            printf("\t""Negatives: ");
+            for (j = 0; negArray[j] != NULL; j++)
+                printf("'%s' ", negArray[j]);
+            printf("\n");
+            
+            boolstuff_free_variables_sets(posArray, negArray);
+        }
+        assert(array[array_size] == NULL);
+        printf("}\n");
 
-	boolstuff_free_node_array(array);
+        boolstuff_free_node_array(array);
     }
 
 
     /*
-	Test tree cloning:
+        Test tree cloning:
     */
 
     dnf_clone = boolstuff_clone_tree(dnf_root);
@@ -232,34 +269,34 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
 
 
     /*
-	Test the parser:
+        Test the parser:
     */
     const char *text = "!(a|b)&(c1_x|!d)";
     parsed_tree = boolstuff_parse(text, &error_index, &error_code);
     if (parsed_tree == NULL)
     {
-	const char *msg = NULL;
+        const char *msg = NULL;
 
-	switch (error_code)
-	{
-	    case BOOLSTUFF_ERR_GARBAGE_AT_END:
-		msg = "garbage at end";
-		break;
-	    case BOOLSTUFF_ERR_RUNAWAY_PARENTHESIS:
-		msg = "runaway parenthesis";
-		break;
-	    case BOOLSTUFF_ERR_IDENTIFIER_EXPECTED:
-		msg = "identifier expected";
-		break;
-	    default:
-		assert(!"unexpected error code");
-	}
+        switch (error_code)
+        {
+            case BOOLSTUFF_ERR_GARBAGE_AT_END:
+                msg = "garbage at end";
+                break;
+            case BOOLSTUFF_ERR_RUNAWAY_PARENTHESIS:
+                msg = "runaway parenthesis";
+                break;
+            case BOOLSTUFF_ERR_IDENTIFIER_EXPECTED:
+                msg = "identifier expected";
+                break;
+            default:
+                assert(!"unexpected error code");
+        }
 
-	printf("Parse error #%d at column %u: %s\n",
-		    error_code, (unsigned) (error_index + 1), msg);
-	printf("Parsed text: \"%s\"\n", text);
+        printf("Parse error #%d at column %u: %s\n",
+                    error_code, (unsigned) (error_index + 1), msg);
+        printf("Parsed text: \"%s\"\n", text);
 
-	return EXIT_FAILURE;
+        return EXIT_FAILURE;
     }
     printf("Parsed: ");
     boolstuff_print_tree(stdout, parsed_tree);
@@ -269,6 +306,8 @@ main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
     assert(TYPE(LEFT(parsed_tree)) == BOOLSTUFF_NOT);
     assert(TYPE(RIGHT(parsed_tree)) == BOOLSTUFF_OR);
 
+    test_foo();
+
     printf("\n""C API tested successfully.\n");
     return EXIT_SUCCESS;
 }
